@@ -22,15 +22,24 @@
 #include <stdlib.h>
 #include <errno.h>
 #include <unistd.h>
+#include <string.h>
 #ifdef ENABLE_HMAC_BINARY_CHECK
 # include <dlfcn.h> 
 #endif
+#ifdef HAVE_SYSLOG
+# include <syslog.h>
+#endif /*HAVE_SYSLOG*/
 
 #include "g10lib.h"
 #include "ath.h"
 #include "cipher-proto.h"
 #include "hmac256.h"
 
+
+/* The name of the file used to foce libgcrypt into fips mode. */
+#define FIPS_FORCE_FILE "/etc/gcrypt/fips_enabled"
+
+
 /* The states of the finite state machine used in fips mode.  */
 enum module_states 
   {
@@ -51,6 +60,9 @@ enum module_states
    fips_mode()! */
 static int no_fips_mode_required;
 
+/* Flag to indicate that we are in the enforced FIPS mode.  */
+static int enforced_fips_mode;
+
 /* This is the lock we use to protect the FSM.  */
 static ath_mutex_t fsm_lock = ATH_MUTEX_INITIALIZER;
 
@@ -60,10 +72,19 @@ static enum module_states current_state;
 
 
 
+
 
 static void fips_new_state (enum module_states new_state);
 
 
+
+/* Convert lowercase hex digits; assumes valid hex digits. */
+#define loxtoi_1(p)   (*(p) <= '9'? (*(p)- '0'): (*(p)-'a'+10))
+#define loxtoi_2(p)   ((loxtoi_1(p) * 16) + loxtoi_1((p)+1))
+
+/* Returns true if P points to a lowercase hex digit. */
+#define loxdigit_p(p) !!strchr ("01234567890abcdef", *(p))
+
 
 
 /* Check whether the OS is in FIPS mode and record that in a module
@@ -90,7 +111,7 @@ _gcry_initialize_fips_mode (int force)
     }
   done = 1;
 
-  /* If the calling applicatione explicitly requested fipsmode, do so.  */
+  /* If the calling application explicitly requested fipsmode, do so.  */
   if (force)
     {
       gcry_assert (!no_fips_mode_required);
@@ -101,11 +122,8 @@ _gcry_initialize_fips_mode (int force)
      provided detection of the FIPS mode and force FIPS mode using a
      file.  The filename is hardwired so that there won't be any
      confusion on whether /etc/gcrypt/ or /usr/local/etc/gcrypt/ is
-     actually used.  The file itself may be empty.  A comment may be
-     included in the file, but comment lines need to be prefixed with
-     a hash mark; only such comment lines and empty lines are
-     allowed.  */
-  if ( !access ("/etc/gcrypt/fips140.force", F_OK) )
+     actually used.  The file itself may be empty.  */
+  if ( !access (FIPS_FORCE_FILE, F_OK) )
     {
       gcry_assert (!no_fips_mode_required);
       goto leave;
@@ -138,6 +156,11 @@ _gcry_initialize_fips_mode (int force)
            file system.  We better stop right away. */
         log_info ("FATAL: error reading `%s' in libgcrypt: %s\n",
                   procfname, strerror (saved_errno));
+#ifdef HAVE_SYSLOG
+        syslog (LOG_USER|LOG_ERR, "Libgcrypt error: "
+                "reading `%s' failed: %s - abort",
+                procfname, strerror (saved_errno));
+#endif /*HAVE_SYSLOG*/
         abort ();
       }
   }
@@ -149,6 +172,7 @@ _gcry_initialize_fips_mode (int force)
   if (!no_fips_mode_required)
     {
       /* Yes, we are in FIPS mode.  */
+      FILE *fp;
 
       /* Intitialize the lock to protect the FSM.  */
       err = ath_mutex_init (&fsm_lock);
@@ -159,9 +183,27 @@ _gcry_initialize_fips_mode (int force)
              get involved.  */
           log_info ("FATAL: failed to create the FSM lock in libgcrypt: %s\n",
                      strerror (err));
+#ifdef HAVE_SYSLOG
+          syslog (LOG_USER|LOG_ERR, "Libgcrypt error: "
+                  "creating FSM lock failed: %s - abort",
+                  strerror (err));
+#endif /*HAVE_SYSLOG*/
           abort ();
         }
+
       
+      /* If the FIPS force files exists, is readable and has a number
+         != 0 on its first line, we enable the enforced fips mode.  */
+      fp = fopen (FIPS_FORCE_FILE, "r");
+      if (fp)
+        {
+          char line[256];
+          
+          if (fgets (line, sizeof line, fp) && atoi (line))
+            enforced_fips_mode = 1;
+          fclose (fp);
+        }
+
       /* Now get us into the INIT state.  */
       fips_new_state (STATE_INIT);
       
@@ -179,6 +221,11 @@ lock_fsm (void)
     {
       log_info ("FATAL: failed to acquire the FSM lock in libgrypt: %s\n", 
                 strerror (err));
+#ifdef HAVE_SYSLOG
+      syslog (LOG_USER|LOG_ERR, "Libgcrypt error: "
+              "acquiring FSM lock failed: %s - abort",
+              strerror (err));
+#endif /*HAVE_SYSLOG*/
       abort ();
     }
 }
@@ -193,6 +240,11 @@ unlock_fsm (void)
     {
       log_info ("FATAL: failed to release the FSM lock in libgrypt: %s\n",
                 strerror (err));
+#ifdef HAVE_SYSLOG
+      syslog (LOG_USER|LOG_ERR, "Libgcrypt error: "
+              "releasing FSM lock failed: %s - abort",
+              strerror (err));
+#endif /*HAVE_SYSLOG*/
       abort ();
     }
 }
@@ -212,6 +264,14 @@ _gcry_fips_mode (void)
 }
 
 
+/* Return a flag telling whether we are in the enforced fips mode.  */
+int 
+_gcry_enforced_fips_mode (void)
+{
+  return enforced_fips_mode;
+}
+
+
 static const char *
 state2str (enum module_states state)
 {
@@ -259,7 +319,7 @@ _gcry_fips_is_operational (void)
              performed by severeal threads; that is no problem because
              our FSM make sure that we won't oversee any error. */
           unlock_fsm ();
-          _gcry_fips_run_selftests ();
+          _gcry_fips_run_selftests (0);
           lock_fsm ();
         }
 
@@ -330,7 +390,7 @@ reporter (const char *domain, int algo, const char *what, const char *errtxt)
 /* Run self-tests for all required cipher algorithms.  Return 0 on
    success. */
 static int
-run_cipher_selftests (void)
+run_cipher_selftests (int extended)
 {
   static int algos[] = 
     {
@@ -346,7 +406,7 @@ run_cipher_selftests (void)
 
   for (idx=0; algos[idx]; idx++)
     {
-      err = _gcry_cipher_selftest (algos[idx], reporter);
+      err = _gcry_cipher_selftest (algos[idx], extended, reporter);
       reporter ("cipher", algos[idx], NULL,
                 err? gpg_strerror (err):NULL);
       if (err)
@@ -359,7 +419,7 @@ run_cipher_selftests (void)
 /* Run self-tests for all required hash algorithms.  Return 0 on
    success. */
 static int
-run_digest_selftests (void)
+run_digest_selftests (int extended)
 {
   static int algos[] = 
     {
@@ -376,7 +436,7 @@ run_digest_selftests (void)
 
   for (idx=0; algos[idx]; idx++)
     {
-      err = _gcry_md_selftest (algos[idx], reporter);
+      err = _gcry_md_selftest (algos[idx], extended, reporter);
       reporter ("digest", algos[idx], NULL,
                 err? gpg_strerror (err):NULL);
       if (err)
@@ -388,7 +448,7 @@ run_digest_selftests (void)
 
 /* Run self-tests for all HMAC algorithms.  Return 0 on success. */
 static int
-run_hmac_selftests (void)
+run_hmac_selftests (int extended)
 {
   static int algos[] = 
     {
@@ -405,7 +465,7 @@ run_hmac_selftests (void)
 
   for (idx=0; algos[idx]; idx++)
     {
-      err = _gcry_hmac_selftest (algos[idx], reporter);
+      err = _gcry_hmac_selftest (algos[idx], extended, reporter);
       reporter ("hmac", algos[idx], NULL,
                 err? gpg_strerror (err):NULL);
       if (err)
@@ -418,7 +478,7 @@ run_hmac_selftests (void)
 /* Run self-tests for all required public key algorithms.  Return 0 on
    success. */
 static int
-run_pubkey_selftests (void)
+run_pubkey_selftests (int extended)
 {
   static int algos[] = 
     {
@@ -433,7 +493,7 @@ run_pubkey_selftests (void)
 
   for (idx=0; algos[idx]; idx++)
     {
-      err = _gcry_pk_selftest (algos[idx], reporter);
+      err = _gcry_pk_selftest (algos[idx], extended, reporter);
       reporter ("pubkey", algos[idx], NULL,
                 err? gpg_strerror (err):NULL);
       if (err)
@@ -480,37 +540,64 @@ check_binary_integrity (void)
         err = gpg_error (GPG_ERR_INTERNAL);
       else
         {
-          FILE *fp;
-  
-          fname = gcry_malloc (strlen (info.dli_fname) + 5 + 1 );
+          fname = gcry_malloc (strlen (info.dli_fname) + 1 + 5 + 1 );
           if (!fname)
             err = gpg_error_from_syserror ();
           else
             {
-              strcpy (stpcpy (fname, info.dli_fname), ".hmac");
-              fp = fopen (fname, "rb");
+              FILE *fp;
+              char *p;
+
+              /* Prefix the basename with a dot.  */
+              strcpy (fname, info.dli_fname);
+              p = strrchr (fname, '/');
+              if (p)
+                p++;
+              else
+                p = fname;
+              memmove (p+1, p, strlen (p)+1);
+              *p = '.';
+              strcat (fname, ".hmac");
+
+              /* Open the file.  */
+              fp = fopen (fname, "r");
               if (!fp)
                 err = gpg_error_from_syserror ();
               else
                 {
-                  char buffer[33];
+                  /* A buffer of 64 bytes plus one for a LF and one to
+                     detect garbage.  */
+                  unsigned char buffer[64+1+1]; 
+                  const unsigned char *s;
                   int n;
 
-                  /* We expect a file of exactly 32 bytes.  Consider
-                     the self-test failed if this is not the case or
-                     if it does not match the just computed HMAC.  */
-                  if ((n=fread (buffer, 1, 33, fp)) != 32
-                      || memcmp (digest, buffer, 32) )
-                    err = gpg_error (GPG_ERR_SELFTEST_FAILED);
-                  else
-                    err = 0;
-
+                  /* The HMAC files consists of lowercase hex digits
+                     only with an optional trailing linefeed.  Fail if
+                     there is any garbage.  */
+                  err = gpg_error (GPG_ERR_SELFTEST_FAILED);
+                  n = fread (buffer, 1, sizeof buffer, fp);
+                  if (n == 64 || (n == 65 && buffer[64] == '\n'))
+                    {
+                      buffer[64] = 0;
+                      for (n=0, s= buffer;
+                           n < 32 && loxdigit_p (s) && loxdigit_p (s+1);
+                           n++, s += 2)
+                        buffer[n] = loxtoi_2 (s);
+                      if ( n == 32 && !memcmp (digest, buffer, 32) )
+                        err = 0;
+                    }
                   fclose (fp);
                 }
             }
         }
     }
   reporter ("binary", 0, fname, err? gpg_strerror (err):NULL);
+#ifdef HAVE_SYSLOG
+  if (err)
+    syslog (LOG_USER|LOG_ERR, "Libgcrypt error: "
+            "integrity check using `%s' failed: %s",
+            fname? fname:"[?]", gpg_strerror (err));
+#endif /*HAVE_SYSLOG*/
   gcry_free (fname);
   return !!err;
 #else
@@ -519,9 +606,10 @@ check_binary_integrity (void)
 }
 
 
-/* Run the self-tests.  */
+/* Run the self-tests.  If EXTENDED is true, extended versions of the
+   selftest are run, that is more tests than required by FIPS.  */
 gpg_err_code_t
-_gcry_fips_run_selftests (void)
+_gcry_fips_run_selftests (int extended)
 {
   enum module_states result = STATE_ERROR;
   gcry_err_code_t ec = GPG_ERR_SELFTEST_FAILED;
@@ -529,21 +617,23 @@ _gcry_fips_run_selftests (void)
   if (fips_mode ())
     fips_new_state (STATE_SELFTEST);
 
-  if (run_cipher_selftests ())
+  if (run_cipher_selftests (extended))
     goto leave;
 
-  if (run_digest_selftests ())
+  if (run_digest_selftests (extended))
     goto leave;
 
-  if (run_hmac_selftests ())
-    goto leave;
-
-  if (run_pubkey_selftests ())
+  if (run_hmac_selftests (extended))
     goto leave;
 
+  /* Run random tests before the pubkey tests because the latter
+     require random.  */
   if (run_random_selftests ())
     goto leave;
 
+  if (run_pubkey_selftests (extended))
+    goto leave;
+
   /* Now check the integrity of the binary.  We do this this after
      having checked the HMAC code.  */
   if (check_binary_integrity ())
@@ -585,6 +675,14 @@ _gcry_fips_signal_error (const char *srcfile, int srcline, const char *srcfunc,
             srcfile, srcline, 
             srcfunc? ", function ":"", srcfunc? srcfunc:"",
             description? description : "no description available");
+#ifdef HAVE_SYSLOG
+  syslog (LOG_USER|LOG_ERR, "Libgcrypt error: "
+          "%serror in file %s, line %d%s%s: %s",
+          is_fatal? "fatal ":"",
+          srcfile, srcline, 
+          srcfunc? ", function ":"", srcfunc? srcfunc:"",
+          description? description : "no description available");
+#endif /*HAVE_SYSLOG*/
 }
 
 
@@ -609,7 +707,9 @@ fips_new_state (enum module_states new_state)
       break;
 
     case STATE_INIT:
-      if (new_state == STATE_SELFTEST )
+      if (new_state == STATE_SELFTEST
+          || new_state == STATE_ERROR
+          || new_state == STATE_FATALERROR)
         ok = 1;
       break;
       
@@ -663,8 +763,21 @@ fips_new_state (enum module_states new_state)
   if (!ok)
     {
       /* Invalid state transition.  Halting library. */
+#ifdef HAVE_SYSLOG
+      syslog (LOG_USER|LOG_ERR, 
+              "Libgcrypt error: invalid state transition %s => %s",
+              state2str (last_state), state2str (new_state));
+#endif /*HAVE_SYSLOG*/
       fips_noreturn ();
     }
+  else if (new_state == STATE_ERROR || new_state == STATE_FATALERROR)
+    {
+#ifdef HAVE_SYSLOG
+      syslog (LOG_USER|LOG_WARNING, 
+              "Libgcrypt notice: state transition %s => %s",
+              state2str (last_state), state2str (new_state));
+#endif /*HAVE_SYSLOG*/
+    }
 }
 
 
@@ -675,6 +788,9 @@ fips_new_state (enum module_states new_state)
 void
 _gcry_fips_noreturn (void)
 {
+#ifdef HAVE_SYSLOG
+  syslog (LOG_USER|LOG_ERR, "Libgcrypt terminated the application");
+#endif /*HAVE_SYSLOG*/
   fflush (NULL);
   abort ();
   /*NOTREACHED*/
