@@ -1375,7 +1375,7 @@ check_one_hmac (int algo, const char *data, int datalen,
   gcry_md_close (hd);
 
   p = gcry_md_read (hd2, algo);
-  if (0 == p)
+  if (!p)
     fail("algo %d, hmac gcry_md_read failed\n", algo);
 
   if (memcmp (p, expect, mdlen))
@@ -2045,6 +2045,7 @@ main (int argc, char **argv)
   int last_argc = -1;
   int debug = 0;
   int use_fips = 0;
+  int selftest_only = 0;
 
   if (argc)
     { argc--; argv++; }
@@ -2072,6 +2073,12 @@ main (int argc, char **argv)
           use_fips = 1;
           argc--; argv++;
         }
+      else if (!strcmp (*argv, "--selftest"))
+        {
+          selftest_only = 1;
+          verbose += 2;
+          argc--; argv++;
+        }
     }          
 
   gcry_control (GCRYCTL_SET_VERBOSITY, (int)verbose);
@@ -2082,30 +2089,35 @@ main (int argc, char **argv)
   if (!gcry_check_version (GCRYPT_VERSION))
     die ("version mismatch\n");
 
+  if ( gcry_fips_mode_active () )
+    in_fips_mode = 1;
+
+  if (!in_fips_mode)
+    gcry_control (GCRYCTL_DISABLE_SECMEM, 0);
+
   if (verbose)
     gcry_set_progress_handler (progress_handler, NULL);
-  
-  gcry_control (GCRYCTL_DISABLE_SECMEM, 0);
+
   gcry_control (GCRYCTL_INITIALIZATION_FINISHED, 0);
   if (debug)
     gcry_control (GCRYCTL_SET_DEBUG_FLAGS, 1u, 0);
   /* No valuable keys are create, so we can speed up our RNG. */
   gcry_control (GCRYCTL_ENABLE_QUICK_RANDOM, 0);
 
-  if ( gcry_control (GCRYCTL_FIPS_MODE_P, 0) )
-    in_fips_mode = 1;
+  if (!selftest_only)
+    {
+      check_ciphers ();
+      check_aes128_cbc_cts_cipher ();
+      check_cbc_mac_cipher ();
+      check_ctr_cipher ();
+      check_cfb_cipher ();
+      check_ofb_cipher ();
+      check_digests ();
+      check_hmac ();
+      check_pubkey ();
+    }
 
-  check_ciphers ();
-  check_aes128_cbc_cts_cipher ();
-  check_cbc_mac_cipher ();
-  check_ctr_cipher ();
-  check_cfb_cipher ();
-  check_ofb_cipher ();
-  check_digests ();
-  check_hmac ();
-  check_pubkey ();
-
-  if (in_fips_mode)
+  if (in_fips_mode && !selftest_only)
     {
       /* If we are in fips mode do some more tests. */
       gcry_md_hd_t md;
@@ -2152,7 +2164,7 @@ main (int argc, char **argv)
   else
     {
       /* If in standard mode, run selftests.  */
-      gcry_control (GCRYCTL_FORCE_FIPS_MODE, 0);
+      gcry_control (GCRYCTL_SELFTEST, 0);
     }
 
   if (verbose)
